@@ -1,4 +1,6 @@
+//#define DEBUG 1
 #include        <string>
+#include        <stack>
 #include        <math.h>
 #include        <stdio.h>
 #include        <ctype.h>
@@ -36,7 +38,8 @@ static gr_font  CurrentFont = {
 }
 
 
-enum position {Superscript, Subscript};	// Indicator
+enum position {Superscript, Subscript, Inline};	// Baseline indicator
+static std::stack<position> pstack;   // baseline position stack
 
 // Use spacing patterned on results of a TeX example (using Large font). All
 // quantities are multiples of Mspace.
@@ -61,8 +64,7 @@ static int      index_for_math_symbol(char *s);	// base routine
 static double   gr_charwidth_cm(int c, int font, double fontsize_pt);
 static void     gr_DrawChar(const char *c);
 static void     gr_setfont_fontsize(gr_fontID newID, bool force = false);
-static void     ClearStack(void);
-static void     PopStack(void);
+static void     pstack_erase();
 static void     MoveDown(void);
 static void     MoveUp(void);
 static void     MoveHorizontally(double em_distance);
@@ -277,8 +279,6 @@ gr_show_at(/*const*/ char *s, double xcm, double ycm, gr_textStyle style, double
 }
 
 // gr_drawstring() -- draw string, including font changes &super/subscripts
-static position pstack[100];
-static int      istack = 0;
 static void
 gr_drawstring(const char *s)
 {
@@ -294,7 +294,6 @@ gr_drawstring(const char *s)
 		return;
 	if (0.0 == gr_currentfontsize_pt())
 		return;
-	ClearStack();
 	// Figure out slant font, if there is an appropriate one
 	switch (original_font) {
 	case gr_font_TimesRoman:
@@ -339,6 +338,9 @@ gr_drawstring(const char *s)
 	// Scan through whole string.
 	START_NEW_TEXT;
 	while (*s != '\0') {
+#ifdef DEBUG
+		printf("DEBUG(%s:%d) *s= '%c'\n",__FILE__,__LINE__,*s);
+#endif
 		if (*s == '-' && CurrentFont.encoding == font_encoding_isolatin1) {
 			// Use a different character to avoid looking like underscore.
 			if (_grWritePS) {
@@ -355,6 +357,11 @@ gr_drawstring(const char *s)
 		// not.
 		if (*s == '$' && slast != '\\') {
 			if (inmath) {
+#ifdef DEBUG
+				printf("DEBUG(%s:%d) got $ so leave math mode\n",__FILE__,__LINE__);
+				printf("DEBUG(%s:%d) pstack size is %d\n",__FILE__,__LINE__,int(pstack.size()));
+#endif
+					
 				// Were in math; now go back to original font.
 				inmath = false;
 				if (current_font != original_font) {
@@ -363,7 +370,14 @@ gr_drawstring(const char *s)
 					gr_setfont(current_font);
 					START_NEW_TEXT;
 				}
+				if (!pstack.empty()) {
+					warning("a text string ended without completing a mathematical grouping (superscript, subscript, or {block})");
+					pstack_erase();
+				}
 			} else {
+#ifdef DEBUG
+				printf("DEBUG(%s:%d) got $ so enter math mode\n",__FILE__,__LINE__);
+#endif
 				// Go to Italic/Oblique font, as case may be.  Unfortunately,
 				// PostScript uses different names for this slanted font.
 				inmath = true;
@@ -373,6 +387,10 @@ gr_drawstring(const char *s)
 					gr_setfont(current_font);
 					START_NEW_TEXT;
 				}
+				if (!pstack.empty()) {
+					warning("a text string started without an empty mathematical grouping (superscript, subscript, or {block})");
+					pstack_erase();
+				}
 			}
 			slast = *s++;
 			continue;
@@ -392,7 +410,10 @@ gr_drawstring(const char *s)
 					return;
 				} else if (*s == '{') {
 					// Several characters to superscript
-					pstack[istack++] = Superscript;
+					pstack.push(Superscript);
+#ifdef DEBUG
+					printf("DEBUG(%s:%d) pushed superscript=%d onto stack to make length %d\n",__FILE__,__LINE__,Superscript, int(pstack.size()));
+#endif
 					MoveUp();
 				} else if (*s == '\\') {
 					// Math character to superscript
@@ -401,7 +422,8 @@ gr_drawstring(const char *s)
 					insert = symbol_in_math(s, &inc);
 					if (inc) {
 						gr_fontID       oldfontID = gr_currentfont();
-						pstack[istack++] = Superscript;
+						pstack.push(Superscript);
+						printf("DEBUG(%s:%d) pushed subscript=%d onto stack to make length %d\n",__FILE__,__LINE__,Superscript, int(pstack.size()));
 						MoveUp();
 						STOP_OLD_TEXT;
 						gr_setfont(gr_font_Symbol);
@@ -412,11 +434,18 @@ gr_drawstring(const char *s)
 						gr_setfont(oldfontID);
 						START_NEW_TEXT;
 						s += inc;
-						PopStack();
+#ifdef DEBUG
+						printf("DEBUG(%s:%d) about to pop stack (was %d) to make length %d\n",__FILE__,__LINE__,pstack.top(), int(pstack.size())-1);
+#endif
+						MoveDown();
+						pstack.pop();
 					}
 				} else {
 					// Single character to superscript
-					pstack[istack++] = Superscript;
+					pstack.push(Superscript);
+#ifdef DEBUG
+					printf("DEBUG(%s:%d) pushed subscript=%d onto stack to make length %d\n",__FILE__,__LINE__,Subscript, int(pstack.size()));
+#endif
 					MoveUp();
 					// Draw single character in math mode.  If it's a digit,
 					// do not do in italics!
@@ -435,7 +464,11 @@ gr_drawstring(const char *s)
 					} else {
 						gr_DrawChar(s);
 					}
-					PopStack();
+					MoveDown();
+#ifdef DEBUG
+					printf("DEBUG(%s:%d) about to pop stack (was %d) to make length %d\n",__FILE__,__LINE__,pstack.top(), int(pstack.size())-1);
+#endif
+					pstack.pop();
 				}
 			} else if (*s == '_') {
 				// Handle subscript
@@ -450,7 +483,10 @@ gr_drawstring(const char *s)
 					return;
 				} else if (*s == '{') {
 					// Several characters to subscript
-					pstack[istack++] = Subscript;
+					pstack.push(Subscript);
+#ifdef DEBUG
+					printf("DEBUG(%s:%d) pushed subscript=%d onto stack to make length %d\n",__FILE__,__LINE__,Subscript, int(pstack.size()));
+#endif
 					MoveDown();
 				} else if (*s == '\\') {
 					// Math character to subscript
@@ -459,7 +495,10 @@ gr_drawstring(const char *s)
 					insert = symbol_in_math(s, &inc);
 					if (inc) {
 						gr_fontID       oldfontID = gr_currentfont();
-						pstack[istack++] = Subscript;
+						pstack.push(Subscript);
+#ifdef DEBUG
+						printf("DEBUG(%s:%d) pushed subscript=%d onto stack to make length %d\n",__FILE__,__LINE__,Subscript, int(pstack.size()));
+#endif
 						MoveDown();
 						STOP_OLD_TEXT;
 						gr_setfont(gr_font_Symbol);
@@ -470,11 +509,18 @@ gr_drawstring(const char *s)
 						gr_setfont(oldfontID);
 						START_NEW_TEXT;
 						s += inc;
-						PopStack();
+						MoveUp();
+#ifdef DEBUG
+						printf("DEBUG(%s:%d) about to pop stack (was %d) to make length %d\n",__FILE__,__LINE__,pstack.top(), int(pstack.size())-1);
+#endif
+						pstack.pop();
 					}
 				} else {
 					// Single character to subscript
-					pstack[istack++] = Subscript;
+					pstack.push(Subscript);
+#ifdef DEBUG
+					printf("DEBUG(%s:%d) pushed subscript=%d onto stack to make length %d\n",__FILE__,__LINE__,Subscript, int(pstack.size()));
+#endif
 					MoveDown();
 					// Draw single character in math mode.  If it's a digit,
 					// do not do in italics!
@@ -493,12 +539,33 @@ gr_drawstring(const char *s)
 					} else {
 						gr_DrawChar(s);
 					}
-					PopStack();
+					MoveUp();
+#ifdef DEBUG
+					printf("DEBUG(%s:%d) about to pop stack (was %d) to make length %d\n",__FILE__,__LINE__,pstack.top(), int(pstack.size())-1);
+#endif
+					pstack.pop();
+				}
+			} else if (*s == '{') { // just a grouping, not a baseline shift
+				pstack.push(Inline);
+#ifdef DEBUG
+				printf("DEBUG(%s:%d) pushed Inline=%d onto stack to make length %d\n",__FILE__,__LINE__,Inline,int(pstack.size()));
+#endif
+
+			} else if (*s == '}') {	// finished with Superscript/Subscript/Inline
+				if (pstack.size() > 0) {
+					position p = pstack.top();
+					if (p == Superscript) {
+						MoveDown();
+					} else if (p == Subscript) {
+						MoveUp();
+					} // ignore inline
+#ifdef DEBUG
+					printf("DEBUG(%s:%d) about to pop stack (was %d) to make length %d\n",__FILE__,__LINE__,pstack.top(), int(pstack.size())-1);
+#endif
+					pstack.pop();
+				} else {
+					warning("unmatched \"}\" in a mathematicsal string");
 				}
-			} else if (*s == '{') {
-				;		// EMPTY
-			} else if (*s == '}') {	// finished with super/sub in math
-				PopStack();
 			} else if (*s == '\\') {
 				// Substitute math symbol, unless it's
 				// an escaped string
@@ -583,6 +650,10 @@ gr_drawstring(const char *s)
 	gr_setfontsize_pt(original_fontsize);
 	gr_setfont(original_font);
 	_drawingstarted = true;
+	if (!pstack.empty()) {
+		warning("a text string ended without completing a mathematical grouping (superscript, subscript, or {block})");
+		pstack_erase();
+	}
 	return;
 }
 
@@ -941,11 +1012,10 @@ static char    *
 symbol_in_math(const char *sPtr, int *inc)
 {
 	/* handle greek letter or symbol in math mode */
-	int             i;
 	sPtr++;
 	*inc = 0;
-	for (i = 0; i < NCODES; i++) {
-		int             len = strlen(symbol_code[i][0]);
+	for (int i = 0; i < NCODES; i++) {
+		int len = strlen(symbol_code[i][0]);
 		if (!strncmp(sPtr, symbol_code[i][0], len)) {
 			*inc = len;
 			return symbol_code[i][1];
@@ -954,22 +1024,26 @@ symbol_in_math(const char *sPtr, int *inc)
 	return NULL;
 }
 
+// Clear the position stack (doesn't STL do this??)
 static void
-PopStack()
+pstack_erase()
 {
-	if (istack > 0 && pstack[istack - 1] == Superscript)
-		MoveDown();
-	else if (istack > 0 && pstack[istack - 1] == Subscript)
-		MoveUp();
-	istack--;
+	while (!pstack.empty()) {
+		position p = pstack.top();
+		if (p == Superscript) {
+			MoveDown();
+		} else if (p == Subscript) {
+			MoveUp();
+		} // ignore inline
+#ifdef DEBUG
+		if (p == Superscript)
+			printf("\tcleared Superscript from position stack\n");
+		else if (p == Subscript)
+			printf("\tcleared Subscript from position stack\n");
+#endif
+		pstack.pop();
+	}
 }
-
-static void
-ClearStack()
-{
-	istack = 0;
-}
-
 // Move left/right by indicated number of M spaces
 static void
 MoveHorizontally(double em_distance)
@@ -985,18 +1059,22 @@ MoveHorizontally(double em_distance)
 static void
 MoveUp()
 {
+#ifdef DEBUG
+	printf("DEBUG(%s:%d) moving text position up one level\n", __FILE__,__LINE__);
+#endif
 	STOP_OLD_TEXT;
 	// See if already in subscript.
-	if ((istack > 0) && pstack[istack - 1] == Subscript) {
+	position p = pstack.top();
+	if (p == Subscript) {
 		// Moving up from subscript, so enlarge font, then undo last move
 		// down.
 		gr_setfontsize_pt(gr_currentfontsize_pt() / SubSize);
 		gr_rmoveto_pt(0.0, SubMoveDown * gr_currentCapHeight_cm() * PT_PER_CM);
 	} else {
-		// Moving up for superscript, so move up, then reduce font.
+		// Moving up from inline or superscript, so move up, then reduce font.
 		gr_rmoveto_pt(0.0, SuperMoveUp * gr_currentCapHeight_cm() * PT_PER_CM);
 		gr_setfontsize_pt(gr_currentfontsize_pt() * SuperSize);
-	}
+	} // ignore Inline
 	START_NEW_TEXT;
 }
 
@@ -1004,15 +1082,18 @@ MoveUp()
 static void
 MoveDown()
 {
+#ifdef DEBUG
+	printf("DEBUG(%s:%d) moving text position down one level\n", __FILE__,__LINE__);
+#endif
 	STOP_OLD_TEXT;
+	position p = pstack.top();
 	// See if already in superscript.
-	if ((istack > 0) && pstack[istack - 1] == Superscript) {
-		// Moving down from superscript, so enlarge font, then undo last move
-		// up.
+	if (p == Superscript) {
+		// Moving down from superscript, so enlarge font, then undo last move up.
 		gr_setfontsize_pt(gr_currentfontsize_pt() / SuperSize);
 		gr_rmoveto_pt(0.0, -SuperMoveUp * gr_currentCapHeight_cm() * PT_PER_CM);
 	} else {
-		// Moving down for subscript, so move down, then reduce font.
+		// Moving down from inline or subscript, so move down, then reduce font.
 		gr_rmoveto_pt(0.0, -SubMoveDown * gr_currentCapHeight_cm() * PT_PER_CM);
 		gr_setfontsize_pt(gr_currentfontsize_pt() * SubSize);
 	}
@@ -1153,7 +1234,7 @@ gr_stringwidth(const char *s, double *w, double *a, double *d)
 				used_supers = true;
 			else if (*s == '_')	// handle subscript
 				used_subs = true;
-			else if (*s == '{')	// ignore groups
+			else if (*s == '{')	// ignore groups while computing string length
 				;		// EMPTY
 			else if (*s == '}')	// ignore groups
 				;		// EMPTY
